@@ -31,7 +31,7 @@ sub add_and_permute {
 	my $coloffset = 0;
 	for (my $i = 0; $i < int($#values / $self->matrix->{n}); $i++) {
 		for (my $j = 0; $j < $self->{matrix}->{n}; $j++) {
-			$self->{matrix}[$j+$i*$self->{matrix}->{n}] = @values[$j+$i*$self->{matrix}->{n}];
+			$self->{matrix}->{mat}[$j+$i*$self->{matrix}->{n}] = @values[$j+$i*$self->{matrix}->{n}];
 			$rowoffset++;
 		}
 		$coloffset++;
@@ -43,8 +43,35 @@ sub add_and_permute {
 	my $backwardoffset = $self->{matrix}->{n} * 2 - ($#values % $self->{matrix}->{n});
 
 	for (my $i = 0; $i < $#values % $self->{matrix}->{n}; $i++) {
-		$self->{matrix}[$rowoffset][$coloffset] = @values[$backwardoffset++]
+		$self->{matrix}->{mat}[$rowoffset][$coloffset] = @values[$backwardoffset++]
 	}
 }
 
+sub permute_row {
+	### substitute row [$i] for [$j]
+	my ($self, $i, $j) = @_;
+
+	my $row = $self->{matrix}->{mat}[$j];
+
+	$self->{matrix}->{mat}[$j] = $self->{matrix}->{mat}[$i];
+	$self->{matrix}->{mat}[$i] = $row;
+}
+
+sub permute_column {
+	### substitute column [$i] for [$j]
+	my ($self, $coli, $colj) = @_;
+
+	my @column = ();		
+
+	for (my $i = 0; $i < $self->{matrix}->{n}; $i++) {
+		push(@column, $self->{matrix}->{mat}[$i][$colj]);
+	}	
+	for (my $i = 0; $i < $self->{matrix}->{n}; $i++) {
+		$self->{matrix}->{mat}[$i][$colj] = $self->{matrix}->{mat}[$i][$coli];
+	} 
+	for (my $i = 0; $i < $self->{matrix}->{n}; $i++}) {
+		$self->{matrix}->{mat}[$i][$coli] = column[$i];
+	} 
+}
+
 1;
